local socket = require'socket'
local gettime = socket and socket.gettime or os.time
local ipairs = ipairs
local unpack = unpack
local insert = table.insert
local remove = table.remove

module ("nsg")

local function add(self, tmo, func, ...)
  tmo = gettime() + tmo
  local timer = {time = tmo, func = func, arg = {...}}
  for i, t in ipairs(self) do
    if t.time > tmo then
      insert(self, i, timer)
      return timer
    end
  end
  insert(self, timer)
  return timer
end

local function del(self, timer)
  if timer then
    for i, t in ipairs(self) do
      if t == timer then remove(self, i) end
    end
  end
end

local function isTimer(self, timer)
  if timer then
    for i, t in ipairs(self) do
      if t == timer then return true end
    end
  end
  return false
end

local function execTimer(timer)
  timer.func(unpack(timer.arg))
end

local function handler(self)
  local t = gettime()
  while true do
    local timer = self[1]
    if not timer then return end
    if timer.time <= t then
      execTimer(remove(self, 1))
    else return end
  end
end

local MAX_TMO = 5
local function nextTmo(self, max_tmo)
  local t = gettime()
  local timer = self[1]
  max_tmo = max_tmo or MAX_TMO
  if not timer or timer.time - t > max_tmo then 
    return max_tmo
  elseif timer.time - t < 0 then
    return 0
  end
  return timer.time - t
end

function timer()
  local timerList = {}
  timerList.add = add
  timerList.del = del
  timerList.isTimer = isTimer
  timerList.handler = handler
  timerList.nextTmo = nextTmo
  
  return timerList
end
